#include <grp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <sys/stat.h>
#include "make_path.h"
#include "get_stat.h"
#include "recurse.h"

static char r_flag;
static char *f_flag = "chgrp";
static char H_flag;
static char v_flag;

static int change(const char *path, void *p) {
	struct group *grp = (struct group *)p;

	struct stat stat_path;
	if (mu_get_stat(f_flag, path, &stat_path))
		return 1;

	if (lchown(path, stat_path.st_uid, grp->gr_gid)) {
		if (f_flag)
			fprintf(stderr, "chgrp: %s: %s\n", path, strerror(errno));

		return 1;
	}

	if (v_flag)
		printf("chgrp: %s: changed group to %s\n", path, grp->gr_name);

	return 0;
}

int main(int argc, char **argv) {

	int opt;
	while ((opt = getopt(argc, argv, "RfHv")) != -1) {
		switch (opt) {
			case 'R':
				r_flag = 1;
				break;

			case 'f':
				f_flag = NULL;
				break;

			case 'H':
				H_flag = 1;
				break;

			case 'v':
				v_flag = 1;
				break;

			default:
				puts("chgrp [RfHv] [group] [file1 file2...]\n\t-H Symbolic link\n\t-R Recursive\n\t-f Silent\n\t-v Verbose");
				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	if (argc == 0) {
		fprintf(stderr, "chgrp: missing operand\n");
		return 1;
	}

	struct group *grp = getgrnam(argv[0]);
	if (!grp) {
		if (f_flag)
			fprintf(stderr, "chgrp: unknow group\n");

		return 1;
	}

	int ret = 0;
	for (int i = 1; i < argc; i++) {
		if (r_flag) {
			if (mu_recurse(f_flag, H_flag, argv[i], (void *)grp, change, change))
				ret = 1;
		}

		else
			ret = change(argv[i], grp);
	}

	return ret;
}
